==> auctioneer-1.0.0.tar.gz: auctioneer/modules/auctioneer_commerce/src/Form/BidAddToCartForm.php <==
<?php

namespace Drupal\auctioneer_commerce\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_order\Form\CustomerFormTrait;
use Drupal\commerce_order\Resolver\OrderTypeResolverInterface;
use Drupal\commerce_price\Price;
use Drupal\auctioneer\Entity\AuctionInterface;
use Drupal\auctioneer\Entity\BidInterface;
use Drupal\auctioneer\AuctionTypeManagerInterface;
use Drupal\auctioneer_commerce\Event\BidAddToCartEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Provides the add to cart form to the winner's one.
 */
class BidAddToCartForm extends FormBase {

  use CustomerFormTrait;

  /**
   * The bid object.
   *
   * @var \Drupal\auctioneer\Entity\BidInterface
   */
  protected $bid;

  /**
   * The user storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $userStorage;

  /**
   * The auction type manager.
   *
   * @var \Drupal\auctioneer\AuctionTypeManagerInterface
   */
  protected $auctionTypeManager;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  /**
   * The cart provider.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * The order type resolver.
   *
   * @var \Drupal\commerce_order\Resolver\OrderTypeResolverInterface
   */
  protected $orderTypeResolver;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    AuctionTypeManagerInterface $auction_type_manager,
    RouteMatchInterface $route_match,
    CartManagerInterface $cart_manager,
    CartProviderInterface $cart_provider,
    OrderTypeResolverInterface $order_type_resolver,
    EventDispatcherInterface $event_dispatcher
  ) {
    $this->userStorage = $auction_type_manager->entityTypeManager()->getStorage('user');
    // Our needed elements itself.
    $this->auctionTypeManager = $auction_type_manager;
    $this->routeMatch = $route_match;
    $this->cartManager = $cart_manager;
    $this->cartProvider = $cart_provider;
    $this->orderTypeResolver = $order_type_resolver;
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('auctioneer.auction_type.manager'),
      $container->get('current_route_match'),
      $container->get('commerce_cart.cart_manager'),
      $container->get('commerce_cart.cart_provider'),
      $container->get('commerce_order.chain_order_type_resolver'),
      $container->get('event_dispatcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'auctioneer_commerce_add_to_cart_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\auctioneer\Entity\BidInterface $bid */
    $bid = $this->routeMatch->getParameter('bid');
    $items = $bid ? $this->getBidItems($bid) : [];
    if (!isset($items['auction']) || !$items['auction'] instanceof AuctionInterface) {
      $form['warning'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('Current bid is not attached to any valid auction.'),
        '#attributes' => [
          'class' => [
            'auction-warning',
            'auction-warning--bid-orphan',
          ],
        ],
      ];

      return $form;
    }
    $settings = $this->getAuctionSettings($items['auction']);
    if (!isset($settings['enable']) || !(bool) $settings['enable']) {
      $form['warning'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t("Current auction type is not enabled to add bids to the owner's cart."),
        '#attributes' => [
          'class' => [
            'auction-warning',
            'auction-warning--auction-type-disabled',
          ],
        ],
      ];

      return $form;
    }
    if (isset($items['store']) && $items['store'] && isset($items['currency']) && $items['currency']) {
      $this->bid = $bid;
      $form = $this->buildBidCustomerForm($form, $form_state);
      $form['summary'] = [
        '#type' => 'view',
        '#name' => 'auctioneer_commerce_cart_summary',
        '#arguments' => [$this->bid->id()],
        '#embed' => TRUE,
        '#prefix' => '<div class="bid-cart-summary">',
        '#suffix' => '</div>',
        '#weight' => -5,
      ];
      $form['actions'] = [
        '#type' => 'actions',
      ];
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t("Add to the owner's cart"),
        '#button_type' => 'primary',
      ];
      if (isset($settings['cart_ajax']) && (bool) $settings['cart_ajax']) {
        $form['ajax_output'] = [
          '#prefix' => '<div class="ajax-output--bid-' . $this->bid->id() . '">',
          '#suffix' => '</div>',
          '#tree' => FALSE,
          '#weight' => -10,
        ];
        $form['actions']['submit']['#ajax'] = [
          'callback' => [$this, 'ajaxCallback'],
        ];
      }
      $form['#access'] = isset($settings['enable']) && (bool) $settings['enable'];
    }
    else {
      $form['warning'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('Current auction does not have any configured Store or Currency settings.'),
        '#attributes' => [
          'class' => [
            'auction-warning',
            'auction-warning--missing-params',
          ],
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    $request = $this->getRequest();
    $response = new AjaxResponse();
    $form['ajax_output']['messages'] = [
      '#type' => 'status_messages',
      '#message_list' => $this->messenger()->all(),
    ];
    $remove_customer_form_command = $disable_submit_form_command = NULL;
    if (!$form_state->hasAnyErrors()) {
      $form['ajax_output']['redirect'] = [
        '#theme' => 'auctioneer_commerce_delay_redirect',
        '#path' => $request->query->get('destination') ?? '',
        '#delay' => 3000,
      ];
      if (isset($form['customer'])) {
        $remove_customer_form_command = new ReplaceCommand('.form-customer--bid-' . $this->bid->id(), '');
      }
      $disable_submit_form_command = new InvokeCommand('.form-actions input', 'disabled', [TRUE]);
    }
    // Let's merge all commands in right order.
    $response->addCommand(new ReplaceCommand('.ajax-output--bid-' . $this->bid->id(), $form['ajax_output']));
    if ($remove_customer_form_command) {
      $response->addCommand($remove_customer_form_command);
    }
    if ($disable_submit_form_command) {
      $response->addCommand($disable_submit_form_command);
    }

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($this->bid) {
      $hammer = $this->auctionTypeManager->getAuctionHammerBidData($this->bid);
      if (empty($hammer)) {
        $form_state->setError($form['actions']['submit'], $this->t('Provided bid is invalid.'));
      }
      if (isset($form['customer'])) {
        $this->validateBidCustomerForm($form, $form_state);
      }
    }
    else {
      $form_state->setError($form['actions']['submit'], $this->t('You must to provide a hammer bid.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $items = $this->bid ? $this->getBidItems($this->bid) : [];
    if (isset($items['auction']) && $items['auction'] && isset($items['store']) && $items['store']) {
      $settings = $this->getAuctionSettings($items['auction']);
      if (isset($settings['enable']) && (bool) $settings['enable'] && isset($settings['bid_amount_field_name']) && $this->bid->hasField($settings['bid_amount_field_name'])) {
        $currency_code = isset($items['currency']) && $items['currency'] ? $items['currency']->getCurrencyCode() : NULL;
        $currency_number = $this->bid->get($settings['bid_amount_field_name'])->first()->getString();
        $commerce_price = $currency_number && $currency_code ? new Price($currency_number, $currency_code) : NULL;
        if (isset($form['customer'])) {
          $this->submitBidCustomerForm($form, $form_state);
        }
        $values = $form_state->getValues();
        $cart_owner = isset($values['uid']) ? $this->userStorage->load($values['uid']) : NULL;
        if ($commerce_price && $cart_owner) {
          $order_item = $this->auctionTypeManager->entityTypeManager()->getStorage('commerce_order_item')->create([
            'type' => 'auctioneer_commerce_auction',
            'quantity' => 1,
            'unit_price' => $commerce_price,
            'title' => $items['auction']->label(),
            'auction_bid' => ['target_id' => $this->bid->id()],
          ]);
          $order_type_id = $this->orderTypeResolver->resolve($order_item);
          $cart = $this->cartProvider->getCart($order_type_id, $items['store'], $cart_owner);
          if (!$cart) {
            $cart = $this->cartProvider->createCart($order_type_id, $items['store'], $cart_owner);
          }
          if ($order_item = $this->cartManager->addOrderItem($cart, $order_item)) {
            $event = new BidAddToCartEvent($this->bid, $order_item);
            $this->eventDispatcher->dispatch(BidAddToCartEvent::EVENT_NAME, $event);
            $this->messenger()->addMessage($this->t("Bid was successfully added to the owner's cart."));
          }
        }
        else {
          $this->messenger()->addMessage($this->t("Can't add to the owner's cart. There was an error fetching needed values."), 'error');
        }
      }
      else {
        $this->messenger()->addMessage($this->t("Can't add to the owner's cart. Invalid bid or provided price amount is not available."), 'error');
      }
    }
    else {
      $this->messenger()->addMessage($this->t("Can't add to the owner's cart. Missing auction or store."), 'error');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildBidCustomerForm(array $form, FormStateInterface $form_state) {
    $items = $this->bid ? $this->getBidItems($this->bid) : [];
    if (isset($items['auction']) && $items['auction']) {
      $settings = $this->getAuctionSettings($items['auction']);
      $uid = isset($items['customer']) && $items['customer'] ? $items['customer']->id() : 0;
      if ($uid > 0 && isset($settings['different_customer']) && (bool) $settings['different_customer']) {
        $form = $this->buildCustomerForm($form, $form_state);
        if (isset($form['customer']) && isset($form['customer']['customer_type'])) {
          $form['customer']['customer_type']['#options']['bid_owner'] = $this->t("Bid's owner");
          if (!$form_state->getTriggeringElement()) {
            $form['customer']['customer_type']['#default_value'] = 'bid_owner';
            $form_state->setValue('customer_type', 'bid_owner');
          }
          if ($form_state->getValue('customer_type') == 'bid_owner') {
            $form['customer']['uid'] = [
              '#type' => 'value',
              '#value' => $uid,
            ];
            $form['customer']['view'] = [
              '#type' => 'view',
              '#name' => 'auctioneer_commerce_cart_customer',
              '#arguments' => [$uid],
              '#embed' => TRUE,
              '#prefix' => '<div class="bid-cart-customer">',
              '#suffix' => '</div>',
            ];
            unset($form['customer']['mail']);
            unset($form['customer']['password']);
            unset($form['customer']['notify']);
          }
        }
      }
      elseif (!($uid > 0)) {
        $form = $this->buildCustomerForm($form, $form_state);
      }
      else {
        $form = [
          'uid' => [
            '#type' => 'value',
            '#value' => $uid,
          ],
          'user' => [
            '#type' => 'fieldset',
            '#title' => $this->t('Customer'),
            'view' => [
              '#type' => 'view',
              '#name' => 'auctioneer_commerce_cart_customer',
              '#arguments' => [$uid],
              '#embed' => TRUE,
              '#prefix' => '<div class="bid-cart-customer">',
              '#suffix' => '</div>',
            ],
          ],
        ];
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateBidCustomerForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if (isset($values['customer_type']) && $values['customer_type'] != 'bid_owner') {
      $this->validateCustomerForm($form, $form_state);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitBidCustomerForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if ($values['customer_type'] != 'bid_owner') {
      $this->submitCustomerForm($form, $form_state);
    }
  }

  /**
   * Get integration settings given an auction entity.
   *
   * @param \Drupal\auctioneer\Entity\AuctionInterface $auction
   *   The auction entity.
   */
  public function getAuctionSettings(AuctionInterface $auction) {
    $settings = $auction->getAuctionType()->getThirdPartySetting('auctioneer_commerce', 'settings');

    return is_array($settings) ? $settings : [];
  }

  /**
   * Get bid required items.
   *
   * @param \Drupal\auctioneer\Entity\BidInterface $bid
   *   The bid entity.
   */
  public function getBidItems(BidInterface $bid) : array {
    $auction = $bid->getAuction();
    $customer = $bid->getOwner();
    $seller = NULL;
    $store = NULL;
    $currency = NULL;
    if ($auction instanceof AuctionInterface) {
      $seller = $auction->getOwner();
      $store = $auction->hasField('auction_commerce_store') ? $auction->get('auction_commerce_store')->entity : NULL;
      $currency = $auction->hasField('auction_commerce_currency') ? $auction->get('auction_commerce_currency')->entity : NULL;
    }

    return [
      'auction' => $auction,
      'seller' => $seller,
      'customer' => $customer,
      'store' => $store,
      'currency' => $currency,
    ];
  }

}

==> auctioneer-2.0.0.tar.gz: auctioneer/modules/auctioneer_commerce/src/Form/BidAddToCartForm.php <==
<?php

namespace Drupal\auctioneer_commerce\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_order\Form\CustomerFormTrait;
use Drupal\commerce_order\Resolver\OrderTypeResolverInterface;
use Drupal\commerce_price\Price;
use Drupal\auctioneer\Entity\AuctionInterface;
use Drupal\auctioneer\Entity\BidInterface;
use Drupal\auctioneer\AuctionTypeManagerInterface;
use Drupal\auctioneer_commerce\Event\BidAddToCartEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * Provides the add to cart form to the winner's one.
 */
class BidAddToCartForm extends FormBase {

  use CustomerFormTrait;

  /**
   * The bid object.
   *
   * @var \Drupal\auctioneer\Entity\BidInterface
   */
  protected $bid;

  /**
   * The user storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $userStorage;

  /**
   * The auction type manager.
   *
   * @var \Drupal\auctioneer\AuctionTypeManagerInterface
   */
  protected $auctionTypeManager;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  /**
   * The cart provider.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * The order type resolver.
   *
   * @var \Drupal\commerce_order\Resolver\OrderTypeResolverInterface
   */
  protected $orderTypeResolver;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Contracts\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    AuctionTypeManagerInterface $auction_type_manager,
    RouteMatchInterface $route_match,
    CartManagerInterface $cart_manager,
    CartProviderInterface $cart_provider,
    OrderTypeResolverInterface $order_type_resolver,
    EventDispatcherInterface $event_dispatcher
  ) {
    $this->userStorage = $auction_type_manager->entityTypeManager()->getStorage('user');
    // Our needed elements itself.
    $this->auctionTypeManager = $auction_type_manager;
    $this->routeMatch = $route_match;
    $this->cartManager = $cart_manager;
    $this->cartProvider = $cart_provider;
    $this->orderTypeResolver = $order_type_resolver;
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('auctioneer.auction_type.manager'),
      $container->get('current_route_match'),
      $container->get('commerce_cart.cart_manager'),
      $container->get('commerce_cart.cart_provider'),
      $container->get('commerce_order.chain_order_type_resolver'),
      $container->get('event_dispatcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'auctioneer_commerce_add_to_cart_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\auctioneer\Entity\BidInterface $bid */
    $bid = $this->routeMatch->getParameter('bid');
    $items = $bid ? $this->getBidItems($bid) : [];
    if (!isset($items['auction']) || !$items['auction'] instanceof AuctionInterface) {
      $form['warning'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('Current bid is not attached to any valid auction.'),
        '#attributes' => [
          'class' => [
            'auction-warning',
            'auction-warning--bid-orphan',
          ],
        ],
      ];

      return $form;
    }
    $settings = $this->getAuctionSettings($items['auction']);
    if (!isset($settings['enable']) || !(bool) $settings['enable']) {
      $form['warning'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t("Current auction type is not enabled to add bids to the owner's cart."),
        '#attributes' => [
          'class' => [
            'auction-warning',
            'auction-warning--auction-type-disabled',
          ],
        ],
      ];

      return $form;
    }
    if (isset($items['store']) && $items['store'] && isset($items['currency']) && $items['currency']) {
      $this->bid = $bid;
      $form = $this->buildBidCustomerForm($form, $form_state);
      $form['summary'] = [
        '#type' => 'view',
        '#name' => 'auctioneer_commerce_cart_summary',
        '#arguments' => [$this->bid->id()],
        '#embed' => TRUE,
        '#prefix' => '<div class="bid-cart-summary">',
        '#suffix' => '</div>',
        '#weight' => -5,
      ];
      $form['actions'] = [
        '#type' => 'actions',
      ];
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t("Add to the owner's cart"),
        '#button_type' => 'primary',
      ];
      if (isset($settings['cart_ajax']) && (bool) $settings['cart_ajax']) {
        $form['ajax_output'] = [
          '#prefix' => '<div class="ajax-output--bid-' . $this->bid->id() . '">',
          '#suffix' => '</div>',
          '#tree' => FALSE,
          '#weight' => -10,
        ];
        $form['actions']['submit']['#ajax'] = [
          'callback' => [$this, 'ajaxCallback'],
        ];
      }
      $form['#access'] = isset($settings['enable']) && (bool) $settings['enable'];
    }
    else {
      $form['warning'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('Current auction does not have any configured Store or Currency settings.'),
        '#attributes' => [
          'class' => [
            'auction-warning',
            'auction-warning--missing-params',
          ],
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    $request = $this->getRequest();
    $response = new AjaxResponse();
    $form['ajax_output']['messages'] = [
      '#type' => 'status_messages',
      '#message_list' => $this->messenger()->all(),
    ];
    $remove_customer_form_command = $disable_submit_form_command = NULL;
    if (!$form_state->hasAnyErrors()) {
      $form['ajax_output']['redirect'] = [
        '#theme' => 'auctioneer_commerce_delay_redirect',
        '#path' => $request->query->get('destination') ?? '',
        '#delay' => 3000,
      ];
      if (isset($form['customer'])) {
        $remove_customer_form_command = new ReplaceCommand('.form-customer--bid-' . $this->bid->id(), '');
      }
      $disable_submit_form_command = new InvokeCommand('.form-actions input', 'disabled', [TRUE]);
    }
    // Let's merge all commands in right order.
    $response->addCommand(new ReplaceCommand('.ajax-output--bid-' . $this->bid->id(), $form['ajax_output']));
    if ($remove_customer_form_command) {
      $response->addCommand($remove_customer_form_command);
    }
    if ($disable_submit_form_command) {
      $response->addCommand($disable_submit_form_command);
    }

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($this->bid) {
      $hammer = $this->auctionTypeManager->getAuctionHammerBidData($this->bid);
      if (empty($hammer)) {
        $form_state->setError($form['actions']['submit'], $this->t('Provided bid is invalid.'));
      }
      if (isset($form['customer'])) {
        $this->validateBidCustomerForm($form, $form_state);
      }
    }
    else {
      $form_state->setError($form['actions']['submit'], $this->t('You must to provide a hammer bid.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $items = $this->bid ? $this->getBidItems($this->bid) : [];
    if (isset($items['auction']) && $items['auction'] && isset($items['store']) && $items['store']) {
      $settings = $this->getAuctionSettings($items['auction']);
      if (isset($settings['enable']) && (bool) $settings['enable'] && isset($settings['bid_amount_field_name']) && $this->bid->hasField($settings['bid_amount_field_name'])) {
        $currency_code = isset($items['currency']) && $items['currency'] ? $items['currency']->getCurrencyCode() : NULL;
        $currency_number = $this->bid->get($settings['bid_amount_field_name'])->first()->getString();
        $commerce_price = $currency_number && $currency_code ? new Price($currency_number, $currency_code) : NULL;
        if (isset($form['customer'])) {
          $this->submitBidCustomerForm($form, $form_state);
        }
        $values = $form_state->getValues();
        $cart_owner = isset($values['uid']) ? $this->userStorage->load($values['uid']) : NULL;
        if ($commerce_price && $cart_owner) {
          $order_item = $this->auctionTypeManager->entityTypeManager()->getStorage('commerce_order_item')->create([
            'type' => 'auctioneer_commerce_auction',
            'quantity' => 1,
            'unit_price' => $commerce_price,
            'title' => $items['auction']->label(),
            'auction_bid' => ['target_id' => $this->bid->id()],
          ]);
          $order_type_id = $this->orderTypeResolver->resolve($order_item);
          $cart = $this->cartProvider->getCart($order_type_id, $items['store'], $cart_owner);
          if (!$cart) {
            $cart = $this->cartProvider->createCart($order_type_id, $items['store'], $cart_owner);
          }
          if ($order_item = $this->cartManager->addOrderItem($cart, $order_item)) {
            $event = new BidAddToCartEvent($this->bid, $order_item);
            $this->eventDispatcher->dispatch($event, BidAddToCartEvent::EVENT_NAME);
            $this->messenger()->addMessage($this->t("Bid was successfully added to the owner's cart."));
          }
        }
        else {
          $this->messenger()->addMessage($this->t("Can't add to the owner's cart. There was an error fetching needed values."), 'error');
        }
      }
      else {
        $this->messenger()->addMessage($this->t("Can't add to the owner's cart. Invalid bid or provided price amount is not available."), 'error');
      }
    }
    else {
      $this->messenger()->addMessage($this->t("Can't add to the owner's cart. Missing auction or store."), 'error');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildBidCustomerForm(array $form, FormStateInterface $form_state) {
    $items = $this->bid ? $this->getBidItems($this->bid) : [];
    if (isset($items['auction']) && $items['auction']) {
      $settings = $this->getAuctionSettings($items['auction']);
      $uid = isset($items['customer']) && $items['customer'] ? $items['customer']->id() : 0;
      if ($uid > 0 && isset($settings['different_customer']) && (bool) $settings['different_customer']) {
        $form = $this->buildCustomerForm($form, $form_state);
        if (isset($form['customer']) && isset($form['customer']['customer_type'])) {
          $form['customer']['customer_type']['#options']['bid_owner'] = $this->t("Bid's owner");
          if (!$form_state->getTriggeringElement()) {
            $form['customer']['customer_type']['#default_value'] = 'bid_owner';
            $form_state->setValue('customer_type', 'bid_owner');
          }
          if ($form_state->getValue('customer_type') == 'bid_owner') {
            $form['customer']['uid'] = [
              '#type' => 'value',
              '#value' => $uid,
            ];
            $form['customer']['view'] = [
              '#type' => 'view',
              '#name' => 'auctioneer_commerce_cart_customer',
              '#arguments' => [$uid],
              '#embed' => TRUE,
              '#prefix' => '<div class="bid-cart-customer">',
              '#suffix' => '</div>',
            ];
            unset($form['customer']['mail']);
            unset($form['customer']['password']);
            unset($form['customer']['notify']);
          }
        }
      }
      elseif (!($uid > 0)) {
        $form = $this->buildCustomerForm($form, $form_state);
      }
      else {
        $form = [
          'uid' => [
            '#type' => 'value',
            '#value' => $uid,
          ],
          'user' => [
            '#type' => 'fieldset',
            '#title' => $this->t('Customer'),
            'view' => [
              '#type' => 'view',
              '#name' => 'auctioneer_commerce_cart_customer',
              '#arguments' => [$uid],
              '#embed' => TRUE,
              '#prefix' => '<div class="bid-cart-customer">',
              '#suffix' => '</div>',
            ],
          ],
        ];
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateBidCustomerForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if (isset($values['customer_type']) && $values['customer_type'] != 'bid_owner') {
      $this->validateCustomerForm($form, $form_state);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitBidCustomerForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if ($values['customer_type'] != 'bid_owner') {
      $this->submitCustomerForm($form, $form_state);
    }
  }

  /**
   * Get integration settings given an auction entity.
   *
   * @param \Drupal\auctioneer\Entity\AuctionInterface $auction
   *   The auction entity.
   */
  public function getAuctionSettings(AuctionInterface $auction) {
    $settings = $auction->getAuctionType()->getThirdPartySetting('auctioneer_commerce', 'settings');

    return is_array($settings) ? $settings : [];
  }

  /**
   * Get bid required items.
   *
   * @param \Drupal\auctioneer\Entity\BidInterface $bid
   *   The bid entity.
   */
  public function getBidItems(BidInterface $bid) : array {
    $auction = $bid->getAuction();
    $customer = $bid->getOwner();
    $seller = NULL;
    $store = NULL;
    $currency = NULL;
    if ($auction instanceof AuctionInterface) {
      $seller = $auction->getOwner();
      $store = $auction->hasField('auction_commerce_store') ? $auction->get('auction_commerce_store')->entity : NULL;
      $currency = $auction->hasField('auction_commerce_currency') ? $auction->get('auction_commerce_currency')->entity : NULL;
    }

    return [
      'auction' => $auction,
      'seller' => $seller,
      'customer' => $customer,
      'store' => $store,
      'currency' => $currency,
    ];
  }

}
